# =============================================================================================
# Copyright 2017 dgketchum
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================================

import os

from rasterio import open as rasopen

from met.thredds import TopoWX, GridMet
from dem import AwsDem
from sat_image.fmask import Fmask


def data_check(model_geo, variable, sat_image=None, temp_units='C'):
    valid_vars = ['tmax', 'tmin', 'dem', 'fmask', 'pet']

    if variable not in valid_vars:
        raise KeyError('Variable {} is invalid, choose from {}'.format(variable,
                                                                       valid_vars))
    file_name = '{}_{}.tif'.format(model_geo.image_id, variable)
    file_path = os.path.join(model_geo.image_dir, file_name)

    if file_name not in os.listdir(model_geo.image_dir):

        if variable == 'tmax':
            var = fetch_temp(model_geo, 'tmax', temp_units, file_path)
        if variable == 'tmin':
            var = fetch_temp(model_geo, 'tmin', temp_units, file_path)
        if variable == 'dem':
            var = fetch_dem(model_geo, file_path)
        if variable == 'fmask':
            var = fetch_fmask(sat_image, file_path)
        if variable == 'pet':
            var = fetch_gridmet(model_geo, 'pet', file_path)

        return var

    else:

        with rasopen(file_path, 'r') as src:
            var = src.read()
            return var


def fetch_gridmet(model_geo, variable='pet', file_path=None):
    gridmet = GridMet(variable, date=model_geo.date,
                      bbox=model_geo.bounds,
                      target_profile=model_geo.profile,
                      clip_feature=model_geo.clip_geo)

    var = gridmet.get_data_subset(out_filename=file_path)

    return var


def fetch_temp(model_geo, variable='tmax', temp_units='C', file_path=None):
    print('Downloading new {}.....'.format(variable))
    topowx = TopoWX(date=model_geo.date, bbox=model_geo.bounds,
                    target_profile=model_geo.profile,
                    clip_feature=model_geo.clip_geo, out_file=file_path)

    var = topowx.get_data_subset(grid_conform=True, var=variable,
                                 out_file=file_path,
                                 temp_units_out=temp_units)
    return var


def fetch_dem(model_geo, file_path=None):
    dem = AwsDem(bounds=model_geo.bounds, clip_object=model_geo.clip_geo,
                 target_profile=model_geo.profile, zoom=8)

    var = dem.terrain(attribute='elevation', out_file=file_path,
                      save_and_return=True)
    return var


def fetch_fmask(sat_image, file_path):

    f = Fmask(sat_image)
    combo = f.cloud_mask(min_filter=(3, 3), max_filter=(40, 40), combined=True)
    f.save_array(combo, file_path)

    return combo


if __name__ == '__main__':
    home = os.path.expanduser('~')

# ========================= EOF ====================================================================



# ===============================================================================
# Copyright 2017 dgketchum
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ===============================================================================
import os
import sys

abspath = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.append(abspath)
import unittest


def suite():

    print('Testing.......................................')

    from tests.test_ssebop import SSEBopModelTestCaseLC8

    loader = unittest.TestLoader()
    test_suite = unittest.TestSuite()

    tests = (SSEBopModelTestCaseLC8,)

    for t in tests:
        test_suite.addTest(loader.loadTestsFromTestCase(t))

    return test_suite


if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(suite())

# ===============================================================================



# ===============================================================================
# Copyright 2017 dgketchum
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ===============================================================================
import os
import sys

abspath = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.append(abspath)
import unittest

from ssebop_app.config import Config
from ssebop_app.paths import paths

from ssebop.ssebop import SSEBopModel
from ssebop.ssebop import SSEBopGeo

from sat_image.image import Landsat8


class SSEBopModelTestCaseLC8(unittest.TestCase):
    def setUp(self):
        self.config_path = os.path.join(os.path.dirname(__file__), 'ssebop_config_test_lc8.yml')
        self.cfg = Config(self.config_path)

    def test_config(self):
        self.assertIsInstance(self.cfg, Config)

    def test_runspecs(self):
        for runspec in self.cfg.runspecs:
            paths.build(runspec.root)
            self.assertEqual(runspec.k_factor, 1.25)

    def test_instantiate_ssebop(self):
        for runspec in self.cfg.runspecs:
            paths.build(runspec.root)
            sseb = SSEBopModel(runspec)
            self.assertIsInstance(sseb, SSEBopModel)
            sseb.configure_run()
            self.assertTrue(sseb._is_configured, True)
            self.assertEqual(runspec.satellite, sseb.satellite)

    def test_image_geo(self):
        for runspec in self.cfg.runspecs:
            paths.build(runspec.root)

            sseb = SSEBopModel(runspec)
            setattr(sseb, 'image', Landsat8(sseb.image_dir))
            image_geo = SSEBopGeo(sseb.image_id, sseb.image_dir, sseb.image.get_tile_geometry(),
                                  sseb.image.transform,
                                  sseb.image.profile, sseb.image.rasterio_geometry)

            self.assertIsInstance(image_geo, SSEBopGeo)
            self.assertIsInstance(image_geo.image_dir, str)

    def test_c_factor(self):
        for runspec in self.cfg.runspecs:
            paths.build(runspec.root)
            sseb = SSEBopModel(runspec)
            sseb.configure_run()
            ts = sseb.image.land_surface_temp()
            c = sseb.c_factor(ts)
            self.assertEqual(c, 1.05873032846)

    def test_difference_temp(self):
        for runspec in self.cfg.runspecs:
            paths.build(runspec.root)
            sseb = SSEBopModel(runspec)
            sseb.configure_run()
            dt = sseb.difference_temp()
            self.assertEqual(dt, 300)


if __name__ == '__main__':
    unittest.main()

# ===============================================================================



# ===============================================================================
# Copyright 2017 ross, dgketchum
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ===============================================================================

from __future__ import print_function

import os
import sys

from dateutil.rrule import rrule, YEARLY


class PathsNotSetExecption(BaseException):
    def __str__(self):
        return 'paths.build(in_root, out_root) needs to be called before the model will run'


class Paths:
    ssebop_root = None
    mask = None
    polygons = None

    def __init__(self):
        self._is_set = False
        self.config = os.path.join(os.path.expanduser('~'), 'ssebop_CONFIG.yml')

    def build(self, parent_root):
        self._is_set = True
        self.ssebop_root = parent_root

    def verify(self):

        if not os.path.exists(self.ssebop_root):
            print('NOT FOUND {}'.format(self.ssebop_root))
            sys.exit(1)

    @staticmethod
    def configure_project_dirs(spec):

        p, r, s = str(spec['path']), str(spec['row']), str(spec['start_date'].year)
        path_row_dir = os.path.join(spec['root'], p, r, s)

        if not os.path.exists(path_row_dir):
            os.mkdir(path_row_dir)
        start, end = spec['start_date'], spec['end_date']

        for dt in rrule(YEARLY, dtstart=start, until=end):
            year_dir = os.path.join(path_row_dir, str(dt.year))
            if not os.path.exists(year_dir):
                os.mkdir(year_dir)

        image_path = spec['image_dir']
        if os.path.exists(image_path):
            if len(os.listdir(image_path)) > 2:
                return True
            else:
                return False
        elif spec['use_existing_images']:
            pass
        else:
            os.mkdir(image_path)
            return False

    def is_set(self):
        return self._is_set


paths = Paths()

# ============= EOF =========================================================



# =============================================================================================
# Copyright 2017 dgketchum
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================================

import os

from ssebop_app.config import Config, paths
from ssebop_app.cli import welcome
from ssebop.ssebop import SSEBopModel


def run_ssebop(cfg_path):
    cfg = Config(cfg_path)
    welcome()
    for runspec in cfg.runspecs:
        paths.build(runspec.root)

        sseb = SSEBopModel(runspec)
        sseb.configure_run()
        sseb.run(overwrite=False)


if __name__ == '__main__':
    home = os.path.expanduser('~')
    root = os.path.join(home, 'PycharmProjects', 'ssebop')
    config_path = os.path.join(root, 'ssebop_config_lc8.yml')
    run_ssebop(config_path)

# ========================= EOF ====================================================================



# =============================================================================================
# Copyright 2017 dgketchum
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================================

from __future__ import print_function

import os
import sys

abspath = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.append(abspath)
from numpy import where, nan, count_nonzero, isnan
from numpy import nanmean, nanstd, deg2rad
from datetime import datetime

from rasterio import open as rasopen
from rasterio.crs import CRS

from ssebop_app.paths import paths, PathsNotSetExecption
from bounds import RasterBounds
from sat_image.image import Landsat5, Landsat7, Landsat8
from ssebop.collector import data_check
from met.fao import get_net_radiation, air_density, air_specific_heat
from met.fao import canopy_resistance
from met.agrimet import Agrimet


class SSEBopModel(object):
    _satellite = None
    _is_configured = False

    def __init__(self, runspec=None, **kwargs):

        self.image = None
        self.dem = None
        self.bounds = None
        self.image_exists = None
        self.use_existing_images = None
        self.image_geo = None

        self.completed = False

        if runspec:
            self.image_dir = runspec.image_dir
            self.parent_dir = runspec.parent_dir
            self.image_exists = runspec.image_exists
            self.image_date = runspec.image_date
            self.satellite = runspec.satellite
            self.path = runspec.path
            self.row = runspec.row
            self.image_id = runspec.image_id
            self.agrimet_corrected = runspec.agrimet_corrected
        else:
            for name, val in kwargs.items():
                setattr(self, name, val)

        if not paths.is_set():
            raise PathsNotSetExecption

        if runspec.verify_paths:
            paths.verify()

        self._info('Constructing/Initializing SSEBop...')

    def configure_run(self):

        self._info('Configuring SSEBop run, checking data...')

        print('----------- CONFIGURATION --------------')
        for attr in ('image_date', 'satellite',
                     'path', 'row', 'image_id', 'image_exists'):
            print('{:<20s}{}'.format(attr, getattr(self, '{}'.format(attr))))
        print('----------- ------------- --------------')

        if not self.image_exists:
            raise NotImplementedError

        self.check_products()

        print('Instantiating image...')

        mapping = {'LT5': Landsat5, 'LE7': Landsat7, 'LC8': Landsat8}
        try:
            cls = mapping[self.satellite]
            self.image = cls(self.image_dir)
        except KeyError:
            print('Invalid satellite key: "{}". available key = {}'.format
                  (self.satellite,
                   ','.join(mapping.keys())))

        self._is_configured = True

        self.image_geo = SSEBopGeo(image_id=self.image_id,
                                   image_dir=self.image_dir,
                                   transform=self.image.rasterio_geometry['affine'],
                                   profile=self.image.rasterio_geometry,
                                   clip_geo=self.image.get_tile_geometry(),
                                   date=self.image_date)

    def run(self, overwrite=False):
        """ Run the SSEBop algorithm.
        :return: 
        """

        if self.completed and not overwrite:
            return None

        dt = self.difference_temp()
        ts = self.image.land_surface_temp()
        c = self.c_factor(ts)
        if not c:
            print('moving to next day due to invalid image for t_corr')
            return None
        ta = data_check(self.image_geo, variable='tmax', temp_units='K')
        tc = c * ta
        th = tc + dt
        etrf = (th - ts) / dt
        pet = data_check(self.image_geo, variable='pet')
        et = pet * etrf
        fmask = data_check(self.image_geo, variable='fmask',
                           sat_image=self.image)
        et_mskd = where(fmask == 0, et, nan)

        self.save_array(et_mskd, variable_name='ssebop_et_mskd',
                        output_path=self.image_dir)
        self.save_array(pet, variable_name='pet', output_path=self.image_dir)
        self.save_array(ts, variable_name='lst', output_path=self.image_dir)
        self.save_array(et, variable_name='ssebop_et', output_path=self.image_dir)
        self.save_array(etrf, variable_name='ssebop_etrf',
                        output_path=self.image_dir)

        if self.agrimet_corrected:
            lat, lon = self.image.scene_coords_deg[0], \
                       self.image.scene_coords_deg[1]
            agrimet = Agrimet(lat=lat, lon=lon)
            # TODO move fetch formed data into instantiation
            # function in both (?) gridmet and agrimet to find bias and correct
        return None

    def c_factor(self, ts):

        ndvi = self.image.ndvi()
        tmax = data_check(self.image_geo, variable='tmax', temp_units='K')
        if len(tmax.shape) > 2:
            tmax = tmax.reshape(tmax.shape[1], tmax.shape[2])

        loc = where(ndvi > 0.7)
        temps = []
        for j, k in zip(loc[0], loc[1]):
            temps.append(tmax[j, k])
        ind = loc[0][0], loc[1][0]

        ta = tmax[ind]
        t_corr_orig = ts / ta
        t_corr_mean = nanmean(t_corr_orig)
        t_diff = ts - ta
        ta = None

        t_corr = where((ndvi >= 0.7) & (ndvi <= 1.0), t_corr_orig, nan)
        ndvi = None
        t_corr_orig = None

        t_corr = where(ts > 270., t_corr, nan)
        ts = None

        t_corr = where((t_diff > 0) & (t_diff < 30), t_corr, nan)
        t_diff = None

        fmask = data_check(self.image_geo, variable='fmask',
                           sat_image=self.image)

        t_corr = where(fmask == 1, t_corr, nan)

        test_count = count_nonzero(~isnan(t_corr))

        if test_count < 50:
            print('Count of clear pixels in {} is insufficient'
                  ' to perform analysis.'.format(self.image_id))
            return None

        print('You have {} pixels for your temperature '
              'correction scheme.'.format(test_count))

        t_corr_std = nanstd(t_corr)
        c = t_corr_mean - (2 * t_corr_std)

        return c

    def difference_temp(self):
        doy = int(datetime.strftime(self.image.date_acquired, '%j'))
        dem = data_check(self.image_geo, variable='dem')
        tmin = data_check(self.image_geo, variable='tmin', temp_units='K')
        tmax = data_check(self.image_geo, variable='tmax', temp_units='K')
        center_lat = (self.image.corner_ll_lat_product + self.image.corner_ul_lat_product) / 2.
        lat_radians = deg2rad(center_lat)
        albedo = self.image.albedo()

        net_rad = get_net_radiation(tmin=tmin, tmax=tmax, doy=doy,
                                    elevation=dem, lat=lat_radians,
                                    albedo=albedo)

        rho = air_density(tmin=tmin, tmax=tmax, elevation=dem)
        cp = air_specific_heat()
        rah = canopy_resistance()

        dt = (net_rad * rah) / (rho * cp)
        # dt = self.image.mask_by_image(arr=dt)
        return dt

    @staticmethod
    def _info(msg):
        print('---------------------------------------')
        print(msg)
        print('---------------------------------------')

    def save_array(self, arr, variable_name, crs=None, output_path=None):

        geometry = self.image.rasterio_geometry

        if not output_path:
            output_filename = os.path.join(self.image_dir,
                                           '{}_{}.tif'.format(self.image_id,
                                                              variable_name))
        else:
            output_filename = os.path.join(output_path,
                                           '{}_{}.tif'.format(self.image_id,
                                                              variable_name))

        try:
            arr = arr.reshape(1, arr.shape[1], arr.shape[2])
        except IndexError:
            arr = arr.reshape(1, arr.shape[0], arr.shape[1])

        geometry['dtype'] = str(arr.dtype)

        if crs:
            geometry['crs'] = CRS({'init': crs})
        with rasopen(output_filename, 'w', **geometry) as dst:
            dst.write(arr)

        return None

    def check_products(self):
        products = ['ssebop_et_mskd', 'pet', 'lst', 'ssebop_et', 'ssebop_etrf']
        for p in products:
            raster = os.path.join(self.image_dir, '{}_{}.tif'.format(self.image_id, p))
            if os.path.isfile(raster):
                print('This analysis has been done for at least {},\n named {} '
                      'use '"overwrite"' parameter'
                      'to overwrite the results or bring them to completion.'.format(p, raster))
                self.completed = True
                return None


class SSEBopGeo:
    def __init__(self, image_id, image_dir, transform,
                 profile, clip_geo, date):
        self.image_id = image_id
        self.image_dir = image_dir
        self.transform = transform
        self.profile = profile
        self.clip_geo = clip_geo
        self.date = date
        self.bounds = RasterBounds(affine_transform=self.transform,
                                   profile=self.profile, latlon=True)

# ========================= EOF ====================================================================



# ===============================================================================
# Copyright 2017 dgketchum
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ===============================================================================

import unittest

try:
    import StringIO
except ImportError:
    from io import StringIO

from ssebop_app.config import Config


class MyTestCase(unittest.TestCase):
    def setUp(self):
        config_text = '''
        input_root: /path/to/data
        output_root: /path/to/output
        start_date: 12/1/2013
        end_date: 12/31/2013
        mask: masks/please_set_me.tif
        polygons: poly/please_set_me.shp
        satellite: LT5
        image_directory: /path/to/landsat_scenes
        esun_table: /path/to/earth_sun_distance_table
        k_factor: 1.25
        dem_folder: /path/to/dem
        use_verify_paths: True
        
        '''
        path = StringIO(config_text)
        self._cfg = Config(path)

    def tearDown(self):
        pass

    def test_run_specs(self):
        self.assertEqual(len(self._cfg.runspecs), 1)

    def test_start_year(self):
        r = self._cfg.runspecs[0]
        s, e = r.date_range
        self.assertEqual(s.year, 2013)

    def test_end_year(self):
        r = self._cfg.runspecs[0]
        s, e = r.date_range
        self.assertEqual(e.year, 2013)

if __name__ == '__main__':
    unittest.main()

# ===============================================================================



# ===============================================================================
# Copyright 2017 ross, dgketchum
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ===============================================================================

from __future__ import print_function

import os
import sys
import click
import logging

# checkout rasterio.rio.options creation_options for mixins todo

from ssebop_app.paths import paths
from ssebop_app.config import Config, check_config
from ssebop.ssebop import SSEBopModel

pp = os.path.realpath(__file__)
sys.path.append(os.path.dirname(os.path.dirname(pp)))

logger = logging.getLogger('ssebop')


@click.group()
def cli():
    pass


@click.command('mkconfig', help='Create a template configuration file')
@click.option('--path', '-p', 'path', default=None, type=click.Path(exists=False, writable=True))
def configure(path):
    """ Create a template configuration file.
    
    :param name: Name of ssebop_config file, :type str
    :param path: Output path and filename for template configuration, if this argument is omitted,
     generic file name will bu used and output saved to root directory. :type str
    :return: None 
    """

    if path:
        click.echo('Creating a config file "{}" '.format(path))
        check_config(path=path)

    else:
        click.echo('Creating a config file "ssebop_config.yml" and sending to home directory')
        check_config(path=None)


@click.command('run', help='Run the SSEBop model')
@click.argument('config_path', default=None, type=click.Path(exists=True))
def run(config_path):
    """ Run the SSEBop model.
    
    :param config_path: Path to a configuration file, if the file does not exist
                     a blank template will be created at your root directory. :type str
    :return: None
    """

    click.echo('Configuration file: {}'.format(config_path))
    click.echo('Running Model')

    cfg = Config(config_path)
    for runspec in cfg.runspecs:
        paths.build(runspec.input_root, runspec.output_root)

        welcome()

        sseb = SSEBopModel(runspec)
        sseb.configure_run(runspec)
        sseb.run()


cli.add_command(configure)
cli.add_command(run)


def welcome():
    print(
        '''
========================================================================

    Welcome to SSEBop 
    
========================================================================
Developed by David Ketchum, 2017
Original Research by Gabriel Senay (USGS); 2007, 2013, 2016

Montana Department of Natural Resources and Conservation

Available commands are enumerated using "ssebop"

For more information regarding a specific command use 
"help <command>". Replace <command>
with the command of interest.
        ''')

# ============= EOF =============================================



# =============================================================================================
# Copyright 2017 dgketchum
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================================


import os

from setuptools import setup

os.environ['TRAVIS_CI'] = 'True'

try:
    from setuptools import setup

    setup_kwargs = {'entry_points': {'console_scripts': ['ssebop=ssebop_app.cli:run']}}
except ImportError:
    from distutils.core import setup

    setup_kwargs = {'scripts': ['bin/ssebop_app/cli']}

with open('README.md') as f:
    readme = f.read()

tag = '0.0.2'

setup(name='ssebop',
      version=tag,
      description='The Operational Simplified Surface Energy Balance',
      setup_requires=[],
      install_requires=['numpy', 'requests', 'netCDF4', 'future',
                        'pandas', 'rasterio', 'fiona'],
      py_modules=['ssebop', 'ssebop_app'],
      license='Apache',
      classifiers=[
          'Development Status :: 3 - Alpha',
          'Intended Audience :: Science/Research',
          'Topic :: Scientific/Engineering :: GIS',
          'License :: OSI Approved :: Apache Software License',
          'Programming Language :: Python :: 3.5'],
      keywords='SSEB operational evapotranspiration algorithm',
      author='David Ketchum (code), Gabriel Senay (algorithm)',
      author_email='dgketchum@gmail.com',
      platforms='Posix; MacOS X; Windows',
      packages=['ssebop', 'ssebop_app', 'tests'],
      download_url='https://github.com/{}/{}/archive/{}.tar.gz'.format('dgketchum', 'ssebop', tag),
      url='https://github.com/dgketchum',
      test_suite='tests.test_suite.suite',
      )

# ========================= EOF ====================================================================



# ===============================================================================
# Copyright 2018 dgketchum
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ===============================================================================

import os
import sys

abspath = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.append(abspath)
from ssebop.ssebop import SSEBopModel


def get_image(image_dir=None, parent_dir=None, image_exists=None,
              image_date=None, satellite=None, path=None, row=None,
              image_id=None):
    spec = {'image_dir': image_dir, 'parent_dir': parent_dir, 'image_exists': image_exists,
            'image_date': image_date, 'satellite': satellite, 'path': path, 'row': row,
            'image_id': image_id}

    sseb = SSEBopModel(spec)
    sseb.configure_run()
    sseb.run(overwrite=False)


if __name__ == '__main__':
    pass
# ========================= EOF ====================================================================



# ===============================================================================
# Copyright 2017 ross, dgketchum
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ===============================================================================

from __future__ import print_function

import os
import sys
from datetime import datetime
from landsat.google_download import GoogleDownload

import yaml

from ssebop_app.paths import paths

DEFAULT_CFG = '''
# SSEBop config file
path: 39
row: 27
root: /home/dgketchum/IrrigationGIS/western_states_irrgis/MT/
output_root: /home/dgketchum/IrrigationGIS/western_states_irrgis/MT/39/27/2013/
satellite: 8
start_date: 20130401
end_date: 20131001
verify_paths: True
agrimet_corrected: True
down_images_only: False
use_existing_images: True
'''

DATETIME_FMT = '%Y%m%d'
JULIAN_FMT = '%Y%j'


class Config:
    _obj = None

    path, row = None, None

    root = None
    api_key = None
    runspecs = None
    satellite = None
    start_date = None
    end_date = None
    usgs_creds = None
    verify_paths = None
    down_images_only = None

    def __init__(self, path=None):
        self.load(path=path)

        p, r, s = str(self.path), str(self.row), str(self.start_date.year)
        self.path_row_dir = os.path.join(self.root, p, r)

        self.set_runspecs()

    def load(self, path=None):
        if path is None:
            path = paths.config

        if isinstance(path, str):
            check_config(path)
            rfile = path
        else:
            rfile = path

        with open(rfile, 'r') as stream:
            try:
                self._obj = yaml.load(stream)
            except yaml.YAMLError as exc:
                print(exc)

            attrs = ('path', 'row', 'root',
                     'start_date', 'end_date',
                     'satellite',
                     'verify_paths',
                     'down_images_only',
                     'agrimet_corrected',
                     'use_existing_images')

            time_attrs = ('start_date', 'end_date')

            for attr in attrs:

                if attr in time_attrs:
                    dt_str = str(self._obj.get(attr))
                    dt = datetime.strptime(dt_str, DATETIME_FMT)
                    setattr(self, attr, dt)

                else:
                    setattr(self, attr, self._obj.get(attr))

    def set_runspecs(self):

        images = self.get_image_list()

        self.runspecs = [RunSpec(image, self) for image in images]

        if self.down_images_only:

            for spec in self.runspecs:
                pass

            self.runspecs = None

    def get_image_list(self, max_cloud_pct=20):

        super_list = []
        s = datetime.strftime(self.start_date, '%Y-%m-%d')
        e = datetime.strftime(self.end_date, '%Y-%m-%d')
        sat_key = int(self.satellite[-1])
        g = GoogleDownload(start=s, end=e, satellite=sat_key,
                           path=self.path, row=self.row, max_cloud_percent=max_cloud_pct)
        images = g.scene_ids
        if images:
            super_list.append(images)
            try:
                flat_list = [item for sublist in super_list for item in sublist]
                flat_list.reverse()
                return flat_list
            except TypeError:
                super_list.reverse()
                return super_list
        else:
            raise AttributeError('No images for this time-frame and satellite....')


class RunSpec(object):
    def __init__(self, image, cfg):
        self.image_id = image
        attrs = ('path', 'row',
                 'satellite',
                 'api_key',
                 'verify_paths',
                 'root',
                 'start_date',
                 'end_date',
                 'down_images_only',
                 'agrimet_corrected',
                 'use_existing_images')

        for attr in attrs:
            cfg_attr = getattr(cfg, attr)
            setattr(self, attr, cfg_attr)

        self.image_date = date = datetime.strptime(image[9:16], JULIAN_FMT)
        self.parent_dir = os.path.join(cfg.path_row_dir, str(date.year))
        self.image_dir = os.path.join(self.parent_dir, image)
        pseudo_spec = {'path': self.path,
                       'row': self.row,
                       'start_date': self.start_date,
                       'end_date': self.end_date,
                       'image_dir': self.image_dir,
                       'root': self.root,
                       'agrimet_corrected': self.agrimet_corrected,
                       'use_existing_images': self.use_existing_images}
        self.image_exists = paths.configure_project_dirs(pseudo_spec)


def check_config(path=None):
    if path is None:
        path = paths.config

    if not os.path.isfile(path):
        print('\n***** The config file {} does not exist. A default one will be written'.format(path))

        with open(path, 'w') as wfile:
            print('-------------- DEFAULT CONFIG -----------------')
            print(DEFAULT_CFG)
            print('-----------------------------------------------')
            wfile.write(DEFAULT_CFG)

        print('***** Please edit the config file at {} and run the model *****\n'.format(
            os.path.join(os.getcwd(), path)))

        sys.exit()

# ============= EOF =============================================
